# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .amount_input_param import AmountInputParam

__all__ = ["PackagePriceInputParam"]


class PackagePriceInputParam(TypedDict, total=False):
    amount: Required[AmountInputParam]

    package_units: Required[int]

    rounding_behavior: Required[Literal["round_up", "round_down"]]

    price_type: Literal["package"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .period import Period as Period
from .period_param import PeriodParam as PeriodParam
from .amount_output import AmountOutput as AmountOutput
from .subject_resource import SubjectResource as SubjectResource
from .flat_price_output import FlatPriceOutput as FlatPriceOutput
from .amount_input_param import AmountInputParam as AmountInputParam
from .rate_card_resource import RateCardResource as RateCardResource
from .invoice_list_params import InvoiceListParams as InvoiceListParams
from .subject_list_params import SubjectListParams as SubjectListParams
from .package_price_output import PackagePriceOutput as PackagePriceOutput
from .invoice_list_response import InvoiceListResponse as InvoiceListResponse
from .rate_card_list_params import RateCardListParams as RateCardListParams
from .subject_create_params import SubjectCreateParams as SubjectCreateParams
from .subject_list_response import SubjectListResponse as SubjectListResponse
from .subject_update_params import SubjectUpdateParams as SubjectUpdateParams
from .subscription_resource import SubscriptionResource as SubscriptionResource
from .flat_price_input_param import FlatPriceInputParam as FlatPriceInputParam
from .checkout_callback_param import CheckoutCallbackParam as CheckoutCallbackParam
from .pricing_metric_resource import PricingMetricResource as PricingMetricResource
from .rate_card_create_params import RateCardCreateParams as RateCardCreateParams
from .rate_card_list_response import RateCardListResponse as RateCardListResponse
from .subject_create_response import SubjectCreateResponse as SubjectCreateResponse
from .subscription_list_params import SubscriptionListParams as SubscriptionListParams
from .package_price_input_param import PackagePriceInputParam as PackagePriceInputParam
from .usage_event_create_params import UsageEventCreateParams as UsageEventCreateParams
from .pricing_metric_list_params import PricingMetricListParams as PricingMetricListParams
from .subscription_cancel_params import SubscriptionCancelParams as SubscriptionCancelParams
from .subscription_create_params import SubscriptionCreateParams as SubscriptionCreateParams
from .subscription_list_response import SubscriptionListResponse as SubscriptionListResponse
from .pricing_metric_create_params import PricingMetricCreateParams as PricingMetricCreateParams
from .pricing_metric_list_response import PricingMetricListResponse as PricingMetricListResponse
from .subscription_create_response import SubscriptionCreateResponse as SubscriptionCreateResponse
from .pricing_metric_create_summary_params import PricingMetricCreateSummaryParams as PricingMetricCreateSummaryParams
from .subscription_change_rate_card_params import SubscriptionChangeRateCardParams as SubscriptionChangeRateCardParams
from .customer_portal_create_session_params import (
    CustomerPortalCreateSessionParams as CustomerPortalCreateSessionParams,
)
from .pricing_metric_create_summary_response import (
    PricingMetricCreateSummaryResponse as PricingMetricCreateSummaryResponse,
)
from .subscription_change_rate_card_response import (
    SubscriptionChangeRateCardResponse as SubscriptionChangeRateCardResponse,
)
from .customer_portal_create_session_response import (
    CustomerPortalCreateSessionResponse as CustomerPortalCreateSessionResponse,
)
from .customer_access_retrieve_billing_state_response import (
    CustomerAccessRetrieveBillingStateResponse as CustomerAccessRetrieveBillingStateResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .flat_price_output import FlatPriceOutput
from .package_price_output import PackagePriceOutput

__all__ = ["RateCardResource", "FixedRate", "FixedRatePrice", "UsageBasedRate", "UsageBasedRatePrice"]

FixedRatePrice: TypeAlias = Annotated[
    Union[FlatPriceOutput, PackagePriceOutput], PropertyInfo(discriminator="price_type")
]


class FixedRate(BaseModel):
    id: str

    description: Optional[str] = None

    name: str

    price: FixedRatePrice
    """Flat price is a price that linearly scales with the quantity."""


UsageBasedRatePrice: TypeAlias = Annotated[
    Union[FlatPriceOutput, PackagePriceOutput], PropertyInfo(discriminator="price_type")
]


class UsageBasedRate(BaseModel):
    id: str

    description: Optional[str] = None

    included_units: int

    name: str

    price: UsageBasedRatePrice
    """Flat price is a price that linearly scales with the quantity."""

    pricing_metric_id: str

    usage_based_rate_type: Optional[Literal["simple"]] = None


class RateCardResource(BaseModel):
    id: str
    """The ID of the rate card."""

    billing_interval: Literal["monthly", "yearly"]

    created_at: datetime
    """The date and time the rate card was created."""

    fixed_rates: List[FixedRate]
    """The fixed rates of the rate card."""

    metadata: Dict[str, str]

    name: str
    """The name of the rate card."""

    updated_at: datetime
    """The date and time the rate card was last updated."""

    usage_based_rates: List[UsageBasedRate]
    """The usage based rates of the rate card."""

    description: Optional[str] = None
    """The description of the rate card."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .period import Period
from .._models import BaseModel

__all__ = ["PricingMetricCreateSummaryResponse"]


class PricingMetricCreateSummaryResponse(BaseModel):
    id: str
    """The ID of the pricing metric summary."""

    period: Period
    """The period that the summary is computed over."""

    pricing_metric_id: str
    """The ID of the pricing metric that the summary is for."""

    subject_id: str
    """The ID of the subject that the summary is for."""

    value: Optional[str] = None
    """The computed value of the pricing metric for the period.

    If the pricing metric does not have any usage events for the period, this will
    be `null`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .checkout_callback_param import CheckoutCallbackParam

__all__ = ["SubscriptionChangeRateCardParams"]


class SubscriptionChangeRateCardParams(TypedDict, total=False):
    checkout_callback_urls: Required[CheckoutCallbackParam]
    """
    The URLs to redirect to after the checkout is completed or cancelled, if a
    checkout is required.
    """

    rate_card_id: Required[str]
    """The ID of the rate card to change the subscription to."""

    upgrade_behavior: Literal["prorate", "rate_difference"]
    """The behavior to use when upgrading the subscription.

    If 'prorate', the customer will be charged for the prorated difference. If
    'rate_difference', the customer will be charged for the difference in the rate
    cards without respect to time.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .subject_resource import SubjectResource

__all__ = ["SubjectListResponse"]


class SubjectListResponse(BaseModel):
    has_more: bool

    subjects: List[SubjectResource]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from lark import Lark, AsyncLark
from lark.types import RateCardResource, RateCardListResponse
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRateCards:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Lark) -> None:
        rate_card = client.rate_cards.create(
            billing_interval="monthly",
            name="Pro Plan",
        )
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Lark) -> None:
        rate_card = client.rate_cards.create(
            billing_interval="monthly",
            name="Pro Plan",
            description="For production applications with moderate usage.",
            fixed_rates=[
                {
                    "name": "Base Rate",
                    "price": {
                        "amount": {
                            "currency_code": "usd",
                            "value": "100",
                        },
                        "price_type": "flat",
                    },
                    "description": None,
                }
            ],
            metadata={"foo": "string"},
            usage_based_rates=[
                {
                    "name": "name",
                    "price": {
                        "amount": {
                            "currency_code": "usd",
                            "value": "2500",
                        },
                        "price_type": "flat",
                    },
                    "pricing_metric_id": "pricing_metric_id",
                    "usage_based_rate_type": "simple",
                    "description": "description",
                    "included_units": 0,
                }
            ],
        )
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Lark) -> None:
        response = client.rate_cards.with_raw_response.create(
            billing_interval="monthly",
            name="Pro Plan",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_card = response.parse()
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Lark) -> None:
        with client.rate_cards.with_streaming_response.create(
            billing_interval="monthly",
            name="Pro Plan",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_card = response.parse()
            assert_matches_type(RateCardResource, rate_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Lark) -> None:
        rate_card = client.rate_cards.retrieve(
            "rate_card_id",
        )
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Lark) -> None:
        response = client.rate_cards.with_raw_response.retrieve(
            "rate_card_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_card = response.parse()
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Lark) -> None:
        with client.rate_cards.with_streaming_response.retrieve(
            "rate_card_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_card = response.parse()
            assert_matches_type(RateCardResource, rate_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Lark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `rate_card_id` but received ''"):
            client.rate_cards.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Lark) -> None:
        rate_card = client.rate_cards.list()
        assert_matches_type(RateCardListResponse, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Lark) -> None:
        rate_card = client.rate_cards.list(
            limit=1,
            offset=0,
        )
        assert_matches_type(RateCardListResponse, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Lark) -> None:
        response = client.rate_cards.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_card = response.parse()
        assert_matches_type(RateCardListResponse, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Lark) -> None:
        with client.rate_cards.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_card = response.parse()
            assert_matches_type(RateCardListResponse, rate_card, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRateCards:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLark) -> None:
        rate_card = await async_client.rate_cards.create(
            billing_interval="monthly",
            name="Pro Plan",
        )
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLark) -> None:
        rate_card = await async_client.rate_cards.create(
            billing_interval="monthly",
            name="Pro Plan",
            description="For production applications with moderate usage.",
            fixed_rates=[
                {
                    "name": "Base Rate",
                    "price": {
                        "amount": {
                            "currency_code": "usd",
                            "value": "100",
                        },
                        "price_type": "flat",
                    },
                    "description": None,
                }
            ],
            metadata={"foo": "string"},
            usage_based_rates=[
                {
                    "name": "name",
                    "price": {
                        "amount": {
                            "currency_code": "usd",
                            "value": "2500",
                        },
                        "price_type": "flat",
                    },
                    "pricing_metric_id": "pricing_metric_id",
                    "usage_based_rate_type": "simple",
                    "description": "description",
                    "included_units": 0,
                }
            ],
        )
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLark) -> None:
        response = await async_client.rate_cards.with_raw_response.create(
            billing_interval="monthly",
            name="Pro Plan",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_card = await response.parse()
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLark) -> None:
        async with async_client.rate_cards.with_streaming_response.create(
            billing_interval="monthly",
            name="Pro Plan",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_card = await response.parse()
            assert_matches_type(RateCardResource, rate_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncLark) -> None:
        rate_card = await async_client.rate_cards.retrieve(
            "rate_card_id",
        )
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncLark) -> None:
        response = await async_client.rate_cards.with_raw_response.retrieve(
            "rate_card_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_card = await response.parse()
        assert_matches_type(RateCardResource, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncLark) -> None:
        async with async_client.rate_cards.with_streaming_response.retrieve(
            "rate_card_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_card = await response.parse()
            assert_matches_type(RateCardResource, rate_card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncLark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `rate_card_id` but received ''"):
            await async_client.rate_cards.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLark) -> None:
        rate_card = await async_client.rate_cards.list()
        assert_matches_type(RateCardListResponse, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLark) -> None:
        rate_card = await async_client.rate_cards.list(
            limit=1,
            offset=0,
        )
        assert_matches_type(RateCardListResponse, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLark) -> None:
        response = await async_client.rate_cards.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_card = await response.parse()
        assert_matches_type(RateCardListResponse, rate_card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLark) -> None:
        async with async_client.rate_cards.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_card = await response.parse()
            assert_matches_type(RateCardListResponse, rate_card, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.customer_access_retrieve_billing_state_response import CustomerAccessRetrieveBillingStateResponse

__all__ = ["CustomerAccessResource", "AsyncCustomerAccessResource"]


class CustomerAccessResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomerAccessResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return CustomerAccessResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomerAccessResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return CustomerAccessResourceWithStreamingResponse(self)

    def retrieve_billing_state(
        self,
        subject_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerAccessRetrieveBillingStateResponse:
        """
        Get Billing State

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subject_id:
            raise ValueError(f"Expected a non-empty value for `subject_id` but received {subject_id!r}")
        return self._get(
            f"/customer-access/{subject_id}/billing-state",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerAccessRetrieveBillingStateResponse,
        )


class AsyncCustomerAccessResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomerAccessResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomerAccessResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomerAccessResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return AsyncCustomerAccessResourceWithStreamingResponse(self)

    async def retrieve_billing_state(
        self,
        subject_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerAccessRetrieveBillingStateResponse:
        """
        Get Billing State

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subject_id:
            raise ValueError(f"Expected a non-empty value for `subject_id` but received {subject_id!r}")
        return await self._get(
            f"/customer-access/{subject_id}/billing-state",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerAccessRetrieveBillingStateResponse,
        )


class CustomerAccessResourceWithRawResponse:
    def __init__(self, customer_access: CustomerAccessResource) -> None:
        self._customer_access = customer_access

        self.retrieve_billing_state = to_raw_response_wrapper(
            customer_access.retrieve_billing_state,
        )


class AsyncCustomerAccessResourceWithRawResponse:
    def __init__(self, customer_access: AsyncCustomerAccessResource) -> None:
        self._customer_access = customer_access

        self.retrieve_billing_state = async_to_raw_response_wrapper(
            customer_access.retrieve_billing_state,
        )


class CustomerAccessResourceWithStreamingResponse:
    def __init__(self, customer_access: CustomerAccessResource) -> None:
        self._customer_access = customer_access

        self.retrieve_billing_state = to_streamed_response_wrapper(
            customer_access.retrieve_billing_state,
        )


class AsyncCustomerAccessResourceWithStreamingResponse:
    def __init__(self, customer_access: AsyncCustomerAccessResource) -> None:
        self._customer_access = customer_access

        self.retrieve_billing_state = async_to_streamed_response_wrapper(
            customer_access.retrieve_billing_state,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["SubscriptionCancelParams"]


class SubscriptionCancelParams(TypedDict, total=False):
    cancel_at_end_of_cycle: Literal[True]
    """Whether to cancel the subscription at end of cycle."""

    reason: Optional[str]
    """The reason for cancelling the subscription."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from lark import Lark, AsyncLark
from lark.types import (
    SubscriptionResource,
    SubscriptionListResponse,
    SubscriptionCreateResponse,
    SubscriptionChangeRateCardResponse,
)
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSubscriptions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Lark) -> None:
        subscription = client.subscriptions.create(
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )
        assert_matches_type(SubscriptionCreateResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Lark) -> None:
        subscription = client.subscriptions.create(
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
            create_checkout_session="when_required",
            metadata={"foo": "string"},
        )
        assert_matches_type(SubscriptionCreateResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Lark) -> None:
        response = client.subscriptions.with_raw_response.create(
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SubscriptionCreateResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Lark) -> None:
        with client.subscriptions.with_streaming_response.create(
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SubscriptionCreateResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Lark) -> None:
        subscription = client.subscriptions.retrieve(
            "subscription_id",
        )
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Lark) -> None:
        response = client.subscriptions.with_raw_response.retrieve(
            "subscription_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Lark) -> None:
        with client.subscriptions.with_streaming_response.retrieve(
            "subscription_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SubscriptionResource, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Lark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            client.subscriptions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Lark) -> None:
        subscription = client.subscriptions.list()
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Lark) -> None:
        subscription = client.subscriptions.list(
            limit=1,
            offset=0,
            rate_card_id="rate_card_id",
            subject_id="subject_id",
        )
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Lark) -> None:
        response = client.subscriptions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Lark) -> None:
        with client.subscriptions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_cancel(self, client: Lark) -> None:
        subscription = client.subscriptions.cancel(
            subscription_id="subscription_id",
        )
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_cancel_with_all_params(self, client: Lark) -> None:
        subscription = client.subscriptions.cancel(
            subscription_id="subscription_id",
            cancel_at_end_of_cycle=True,
            reason="user_requested",
        )
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_cancel(self, client: Lark) -> None:
        response = client.subscriptions.with_raw_response.cancel(
            subscription_id="subscription_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_cancel(self, client: Lark) -> None:
        with client.subscriptions.with_streaming_response.cancel(
            subscription_id="subscription_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SubscriptionResource, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_cancel(self, client: Lark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            client.subscriptions.with_raw_response.cancel(
                subscription_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_change_rate_card(self, client: Lark) -> None:
        subscription = client.subscriptions.change_rate_card(
            subscription_id="subscription_id",
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
        )
        assert_matches_type(SubscriptionChangeRateCardResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_change_rate_card_with_all_params(self, client: Lark) -> None:
        subscription = client.subscriptions.change_rate_card(
            subscription_id="subscription_id",
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
            upgrade_behavior="prorate",
        )
        assert_matches_type(SubscriptionChangeRateCardResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_change_rate_card(self, client: Lark) -> None:
        response = client.subscriptions.with_raw_response.change_rate_card(
            subscription_id="subscription_id",
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SubscriptionChangeRateCardResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_change_rate_card(self, client: Lark) -> None:
        with client.subscriptions.with_streaming_response.change_rate_card(
            subscription_id="subscription_id",
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SubscriptionChangeRateCardResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_change_rate_card(self, client: Lark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            client.subscriptions.with_raw_response.change_rate_card(
                subscription_id="",
                checkout_callback_urls={
                    "cancelled_url": "https://example.com/callback",
                    "success_url": "https://example.com/callback",
                },
                rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
            )


class TestAsyncSubscriptions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLark) -> None:
        subscription = await async_client.subscriptions.create(
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )
        assert_matches_type(SubscriptionCreateResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLark) -> None:
        subscription = await async_client.subscriptions.create(
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
            create_checkout_session="when_required",
            metadata={"foo": "string"},
        )
        assert_matches_type(SubscriptionCreateResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLark) -> None:
        response = await async_client.subscriptions.with_raw_response.create(
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(SubscriptionCreateResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLark) -> None:
        async with async_client.subscriptions.with_streaming_response.create(
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_AJWMxR81jxoRlli6p13uf3JB",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(SubscriptionCreateResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncLark) -> None:
        subscription = await async_client.subscriptions.retrieve(
            "subscription_id",
        )
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncLark) -> None:
        response = await async_client.subscriptions.with_raw_response.retrieve(
            "subscription_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncLark) -> None:
        async with async_client.subscriptions.with_streaming_response.retrieve(
            "subscription_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(SubscriptionResource, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncLark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            await async_client.subscriptions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLark) -> None:
        subscription = await async_client.subscriptions.list()
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLark) -> None:
        subscription = await async_client.subscriptions.list(
            limit=1,
            offset=0,
            rate_card_id="rate_card_id",
            subject_id="subject_id",
        )
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLark) -> None:
        response = await async_client.subscriptions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLark) -> None:
        async with async_client.subscriptions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_cancel(self, async_client: AsyncLark) -> None:
        subscription = await async_client.subscriptions.cancel(
            subscription_id="subscription_id",
        )
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_cancel_with_all_params(self, async_client: AsyncLark) -> None:
        subscription = await async_client.subscriptions.cancel(
            subscription_id="subscription_id",
            cancel_at_end_of_cycle=True,
            reason="user_requested",
        )
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncLark) -> None:
        response = await async_client.subscriptions.with_raw_response.cancel(
            subscription_id="subscription_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(SubscriptionResource, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncLark) -> None:
        async with async_client.subscriptions.with_streaming_response.cancel(
            subscription_id="subscription_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(SubscriptionResource, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncLark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            await async_client.subscriptions.with_raw_response.cancel(
                subscription_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_change_rate_card(self, async_client: AsyncLark) -> None:
        subscription = await async_client.subscriptions.change_rate_card(
            subscription_id="subscription_id",
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
        )
        assert_matches_type(SubscriptionChangeRateCardResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_change_rate_card_with_all_params(self, async_client: AsyncLark) -> None:
        subscription = await async_client.subscriptions.change_rate_card(
            subscription_id="subscription_id",
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
            upgrade_behavior="prorate",
        )
        assert_matches_type(SubscriptionChangeRateCardResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_change_rate_card(self, async_client: AsyncLark) -> None:
        response = await async_client.subscriptions.with_raw_response.change_rate_card(
            subscription_id="subscription_id",
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(SubscriptionChangeRateCardResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_change_rate_card(self, async_client: AsyncLark) -> None:
        async with async_client.subscriptions.with_streaming_response.change_rate_card(
            subscription_id="subscription_id",
            checkout_callback_urls={
                "cancelled_url": "https://example.com/callback",
                "success_url": "https://example.com/callback",
            },
            rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(SubscriptionChangeRateCardResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_change_rate_card(self, async_client: AsyncLark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            await async_client.subscriptions.with_raw_response.change_rate_card(
                subscription_id="",
                checkout_callback_urls={
                    "cancelled_url": "https://example.com/callback",
                    "success_url": "https://example.com/callback",
                },
                rate_card_id="rc_jQK2n0wutCj6bBcAIrL6o07g",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from lark import Lark, AsyncLark
from lark._utils import parse_datetime
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsageEvents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Lark) -> None:
        usage_event = client.usage_events.create(
            data={
                "compute_hours": 100,
                "instance_type": "t2.micro",
                "region": "us-east-1",
            },
            event_name="compute_hours",
            idempotency_key="dd187757-47af-4a69-8a89-6285b9fe1cb6",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )
        assert_matches_type(object, usage_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Lark) -> None:
        usage_event = client.usage_events.create(
            data={
                "compute_hours": 100,
                "instance_type": "t2.micro",
                "region": "us-east-1",
            },
            event_name="compute_hours",
            idempotency_key="dd187757-47af-4a69-8a89-6285b9fe1cb6",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
            timestamp=parse_datetime("2025-11-13T19:58:33.898142+00:00"),
        )
        assert_matches_type(object, usage_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Lark) -> None:
        response = client.usage_events.with_raw_response.create(
            data={
                "compute_hours": 100,
                "instance_type": "t2.micro",
                "region": "us-east-1",
            },
            event_name="compute_hours",
            idempotency_key="dd187757-47af-4a69-8a89-6285b9fe1cb6",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage_event = response.parse()
        assert_matches_type(object, usage_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Lark) -> None:
        with client.usage_events.with_streaming_response.create(
            data={
                "compute_hours": 100,
                "instance_type": "t2.micro",
                "region": "us-east-1",
            },
            event_name="compute_hours",
            idempotency_key="dd187757-47af-4a69-8a89-6285b9fe1cb6",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage_event = response.parse()
            assert_matches_type(object, usage_event, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUsageEvents:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLark) -> None:
        usage_event = await async_client.usage_events.create(
            data={
                "compute_hours": 100,
                "instance_type": "t2.micro",
                "region": "us-east-1",
            },
            event_name="compute_hours",
            idempotency_key="dd187757-47af-4a69-8a89-6285b9fe1cb6",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )
        assert_matches_type(object, usage_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLark) -> None:
        usage_event = await async_client.usage_events.create(
            data={
                "compute_hours": 100,
                "instance_type": "t2.micro",
                "region": "us-east-1",
            },
            event_name="compute_hours",
            idempotency_key="dd187757-47af-4a69-8a89-6285b9fe1cb6",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
            timestamp=parse_datetime("2025-11-13T19:58:33.898142+00:00"),
        )
        assert_matches_type(object, usage_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLark) -> None:
        response = await async_client.usage_events.with_raw_response.create(
            data={
                "compute_hours": 100,
                "instance_type": "t2.micro",
                "region": "us-east-1",
            },
            event_name="compute_hours",
            idempotency_key="dd187757-47af-4a69-8a89-6285b9fe1cb6",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage_event = await response.parse()
        assert_matches_type(object, usage_event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLark) -> None:
        async with async_client.usage_events.with_streaming_response.create(
            data={
                "compute_hours": 100,
                "instance_type": "t2.micro",
                "region": "us-east-1",
            },
            event_name="compute_hours",
            idempotency_key="dd187757-47af-4a69-8a89-6285b9fe1cb6",
            subject_id="subj_VyX6Q96h5avMho8O7QWlKeXE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage_event = await response.parse()
            assert_matches_type(object, usage_event, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

from .checkout_callback_param import CheckoutCallbackParam

__all__ = ["SubscriptionCreateParams"]


class SubscriptionCreateParams(TypedDict, total=False):
    checkout_callback_urls: Required[CheckoutCallbackParam]
    """
    The URLs to redirect to after the checkout is completed or cancelled, if a
    checkout is required.
    """

    rate_card_id: Required[str]
    """The ID of the rate card to use for the subscription."""

    subject_id: Required[str]
    """The ID or external ID of the subject to create the subscription for."""

    create_checkout_session: Literal["when_required", "always"]
    """
    Determines whether a checkout session is always required even if the subject has
    a payment method on file. By default, if the subject has a payment method on
    file or the subscription is for a free plan, the subscription will be created
    and billed for immediately (if for a paid plan).
    """

    metadata: Dict[str, str]
    """Additional metadata about the subscription.

    You may use this to store any custom data about the subscription.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from ..types import (
    subscription_list_params,
    subscription_cancel_params,
    subscription_create_params,
    subscription_change_rate_card_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.subscription_resource import SubscriptionResource
from ..types.checkout_callback_param import CheckoutCallbackParam
from ..types.subscription_list_response import SubscriptionListResponse
from ..types.subscription_create_response import SubscriptionCreateResponse
from ..types.subscription_change_rate_card_response import SubscriptionChangeRateCardResponse

__all__ = ["SubscriptionsResource", "AsyncSubscriptionsResource"]


class SubscriptionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return SubscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return SubscriptionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        checkout_callback_urls: CheckoutCallbackParam,
        rate_card_id: str,
        subject_id: str,
        create_checkout_session: Literal["when_required", "always"] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionCreateResponse:
        """
        Create Subscription

        Args:
          checkout_callback_urls: The URLs to redirect to after the checkout is completed or cancelled, if a
              checkout is required.

          rate_card_id: The ID of the rate card to use for the subscription.

          subject_id: The ID or external ID of the subject to create the subscription for.

          create_checkout_session: Determines whether a checkout session is always required even if the subject has
              a payment method on file. By default, if the subject has a payment method on
              file or the subscription is for a free plan, the subscription will be created
              and billed for immediately (if for a paid plan).

          metadata: Additional metadata about the subscription. You may use this to store any custom
              data about the subscription.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/subscriptions",
            body=maybe_transform(
                {
                    "checkout_callback_urls": checkout_callback_urls,
                    "rate_card_id": rate_card_id,
                    "subject_id": subject_id,
                    "create_checkout_session": create_checkout_session,
                    "metadata": metadata,
                },
                subscription_create_params.SubscriptionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubscriptionCreateResponse,
        )

    def retrieve(
        self,
        subscription_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionResource:
        """
        Get Subscription

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return self._get(
            f"/subscriptions/{subscription_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubscriptionResource,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        rate_card_id: Optional[str] | Omit = omit,
        subject_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionListResponse:
        """
        List Subscriptions

        Args:
          rate_card_id: The ID of the rate card to list subscriptions for. Cannot be used with
              subject_id.

          subject_id: The ID or external ID of the subject to list subscriptions for. Cannot be used
              with rate_card_id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/subscriptions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "rate_card_id": rate_card_id,
                        "subject_id": subject_id,
                    },
                    subscription_list_params.SubscriptionListParams,
                ),
            ),
            cast_to=SubscriptionListResponse,
        )

    def cancel(
        self,
        subscription_id: str,
        *,
        cancel_at_end_of_cycle: Literal[True] | Omit = omit,
        reason: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionResource:
        """
        Cancel Subscription

        Args:
          cancel_at_end_of_cycle: Whether to cancel the subscription at end of cycle.

          reason: The reason for cancelling the subscription.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return self._post(
            f"/subscriptions/{subscription_id}/cancel",
            body=maybe_transform(
                {
                    "cancel_at_end_of_cycle": cancel_at_end_of_cycle,
                    "reason": reason,
                },
                subscription_cancel_params.SubscriptionCancelParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubscriptionResource,
        )

    def change_rate_card(
        self,
        subscription_id: str,
        *,
        checkout_callback_urls: CheckoutCallbackParam,
        rate_card_id: str,
        upgrade_behavior: Literal["prorate", "rate_difference"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionChangeRateCardResponse:
        """
        Change Subscription Rate Card

        Args:
          checkout_callback_urls: The URLs to redirect to after the checkout is completed or cancelled, if a
              checkout is required.

          rate_card_id: The ID of the rate card to change the subscription to.

          upgrade_behavior: The behavior to use when upgrading the subscription. If 'prorate', the customer
              will be charged for the prorated difference. If 'rate_difference', the customer
              will be charged for the difference in the rate cards without respect to time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return self._post(
            f"/subscriptions/{subscription_id}/change-rate-card",
            body=maybe_transform(
                {
                    "checkout_callback_urls": checkout_callback_urls,
                    "rate_card_id": rate_card_id,
                    "upgrade_behavior": upgrade_behavior,
                },
                subscription_change_rate_card_params.SubscriptionChangeRateCardParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubscriptionChangeRateCardResponse,
        )


class AsyncSubscriptionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return AsyncSubscriptionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        checkout_callback_urls: CheckoutCallbackParam,
        rate_card_id: str,
        subject_id: str,
        create_checkout_session: Literal["when_required", "always"] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionCreateResponse:
        """
        Create Subscription

        Args:
          checkout_callback_urls: The URLs to redirect to after the checkout is completed or cancelled, if a
              checkout is required.

          rate_card_id: The ID of the rate card to use for the subscription.

          subject_id: The ID or external ID of the subject to create the subscription for.

          create_checkout_session: Determines whether a checkout session is always required even if the subject has
              a payment method on file. By default, if the subject has a payment method on
              file or the subscription is for a free plan, the subscription will be created
              and billed for immediately (if for a paid plan).

          metadata: Additional metadata about the subscription. You may use this to store any custom
              data about the subscription.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/subscriptions",
            body=await async_maybe_transform(
                {
                    "checkout_callback_urls": checkout_callback_urls,
                    "rate_card_id": rate_card_id,
                    "subject_id": subject_id,
                    "create_checkout_session": create_checkout_session,
                    "metadata": metadata,
                },
                subscription_create_params.SubscriptionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubscriptionCreateResponse,
        )

    async def retrieve(
        self,
        subscription_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionResource:
        """
        Get Subscription

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return await self._get(
            f"/subscriptions/{subscription_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubscriptionResource,
        )

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        rate_card_id: Optional[str] | Omit = omit,
        subject_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionListResponse:
        """
        List Subscriptions

        Args:
          rate_card_id: The ID of the rate card to list subscriptions for. Cannot be used with
              subject_id.

          subject_id: The ID or external ID of the subject to list subscriptions for. Cannot be used
              with rate_card_id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/subscriptions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "rate_card_id": rate_card_id,
                        "subject_id": subject_id,
                    },
                    subscription_list_params.SubscriptionListParams,
                ),
            ),
            cast_to=SubscriptionListResponse,
        )

    async def cancel(
        self,
        subscription_id: str,
        *,
        cancel_at_end_of_cycle: Literal[True] | Omit = omit,
        reason: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionResource:
        """
        Cancel Subscription

        Args:
          cancel_at_end_of_cycle: Whether to cancel the subscription at end of cycle.

          reason: The reason for cancelling the subscription.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return await self._post(
            f"/subscriptions/{subscription_id}/cancel",
            body=await async_maybe_transform(
                {
                    "cancel_at_end_of_cycle": cancel_at_end_of_cycle,
                    "reason": reason,
                },
                subscription_cancel_params.SubscriptionCancelParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubscriptionResource,
        )

    async def change_rate_card(
        self,
        subscription_id: str,
        *,
        checkout_callback_urls: CheckoutCallbackParam,
        rate_card_id: str,
        upgrade_behavior: Literal["prorate", "rate_difference"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubscriptionChangeRateCardResponse:
        """
        Change Subscription Rate Card

        Args:
          checkout_callback_urls: The URLs to redirect to after the checkout is completed or cancelled, if a
              checkout is required.

          rate_card_id: The ID of the rate card to change the subscription to.

          upgrade_behavior: The behavior to use when upgrading the subscription. If 'prorate', the customer
              will be charged for the prorated difference. If 'rate_difference', the customer
              will be charged for the difference in the rate cards without respect to time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return await self._post(
            f"/subscriptions/{subscription_id}/change-rate-card",
            body=await async_maybe_transform(
                {
                    "checkout_callback_urls": checkout_callback_urls,
                    "rate_card_id": rate_card_id,
                    "upgrade_behavior": upgrade_behavior,
                },
                subscription_change_rate_card_params.SubscriptionChangeRateCardParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubscriptionChangeRateCardResponse,
        )


class SubscriptionsResourceWithRawResponse:
    def __init__(self, subscriptions: SubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = to_raw_response_wrapper(
            subscriptions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            subscriptions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            subscriptions.list,
        )
        self.cancel = to_raw_response_wrapper(
            subscriptions.cancel,
        )
        self.change_rate_card = to_raw_response_wrapper(
            subscriptions.change_rate_card,
        )


class AsyncSubscriptionsResourceWithRawResponse:
    def __init__(self, subscriptions: AsyncSubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = async_to_raw_response_wrapper(
            subscriptions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            subscriptions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            subscriptions.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            subscriptions.cancel,
        )
        self.change_rate_card = async_to_raw_response_wrapper(
            subscriptions.change_rate_card,
        )


class SubscriptionsResourceWithStreamingResponse:
    def __init__(self, subscriptions: SubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = to_streamed_response_wrapper(
            subscriptions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            subscriptions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            subscriptions.list,
        )
        self.cancel = to_streamed_response_wrapper(
            subscriptions.cancel,
        )
        self.change_rate_card = to_streamed_response_wrapper(
            subscriptions.change_rate_card,
        )


class AsyncSubscriptionsResourceWithStreamingResponse:
    def __init__(self, subscriptions: AsyncSubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = async_to_streamed_response_wrapper(
            subscriptions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            subscriptions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            subscriptions.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            subscriptions.cancel,
        )
        self.change_rate_card = async_to_streamed_response_wrapper(
            subscriptions.change_rate_card,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from ..types import subject_list_params, subject_create_params, subject_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.subject_resource import SubjectResource
from ..types.subject_list_response import SubjectListResponse
from ..types.subject_create_response import SubjectCreateResponse

__all__ = ["SubjectsResource", "AsyncSubjectsResource"]


class SubjectsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return SubjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return SubjectsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        email: Optional[str] | Omit = omit,
        external_id: Optional[str] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubjectCreateResponse:
        """Create Subject

        Args:
          email: The email of the subject.

        Must be a valid email address.

          external_id: The ID of the subject in your system. If provided, you may use pass it to the
              API in place of the subject ID. Must be unique.

          metadata: Additional metadata about the subject. You may use this to store any custom data
              about the subject.

          name: The name of the subject. Used for display in the dashboard.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/subjects",
            body=maybe_transform(
                {
                    "email": email,
                    "external_id": external_id,
                    "metadata": metadata,
                    "name": name,
                },
                subject_create_params.SubjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubjectCreateResponse,
        )

    def retrieve(
        self,
        subject_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubjectResource:
        """
        Get Subject

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subject_id:
            raise ValueError(f"Expected a non-empty value for `subject_id` but received {subject_id!r}")
        return self._get(
            f"/subjects/{subject_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubjectResource,
        )

    def update(
        self,
        subject_id: str,
        *,
        email: Optional[str],
        metadata: Optional[Dict[str, str]],
        name: Optional[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubjectResource:
        """Update Subject

        Args:
          email: The email of the subject.

        Must be a valid email address.

          metadata: Additional metadata about the subject. You may use this to store any custom data
              about the subject.

          name: The name of the subject. Used for display in the dashboard.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subject_id:
            raise ValueError(f"Expected a non-empty value for `subject_id` but received {subject_id!r}")
        return self._put(
            f"/subjects/{subject_id}",
            body=maybe_transform(
                {
                    "email": email,
                    "metadata": metadata,
                    "name": name,
                },
                subject_update_params.SubjectUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubjectResource,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubjectListResponse:
        """
        List Subjects

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/subjects",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    subject_list_params.SubjectListParams,
                ),
            ),
            cast_to=SubjectListResponse,
        )

    def delete(
        self,
        subject_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Subject

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subject_id:
            raise ValueError(f"Expected a non-empty value for `subject_id` but received {subject_id!r}")
        return self._delete(
            f"/subjects/{subject_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncSubjectsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return AsyncSubjectsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        email: Optional[str] | Omit = omit,
        external_id: Optional[str] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubjectCreateResponse:
        """Create Subject

        Args:
          email: The email of the subject.

        Must be a valid email address.

          external_id: The ID of the subject in your system. If provided, you may use pass it to the
              API in place of the subject ID. Must be unique.

          metadata: Additional metadata about the subject. You may use this to store any custom data
              about the subject.

          name: The name of the subject. Used for display in the dashboard.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/subjects",
            body=await async_maybe_transform(
                {
                    "email": email,
                    "external_id": external_id,
                    "metadata": metadata,
                    "name": name,
                },
                subject_create_params.SubjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubjectCreateResponse,
        )

    async def retrieve(
        self,
        subject_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubjectResource:
        """
        Get Subject

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subject_id:
            raise ValueError(f"Expected a non-empty value for `subject_id` but received {subject_id!r}")
        return await self._get(
            f"/subjects/{subject_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubjectResource,
        )

    async def update(
        self,
        subject_id: str,
        *,
        email: Optional[str],
        metadata: Optional[Dict[str, str]],
        name: Optional[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubjectResource:
        """Update Subject

        Args:
          email: The email of the subject.

        Must be a valid email address.

          metadata: Additional metadata about the subject. You may use this to store any custom data
              about the subject.

          name: The name of the subject. Used for display in the dashboard.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subject_id:
            raise ValueError(f"Expected a non-empty value for `subject_id` but received {subject_id!r}")
        return await self._put(
            f"/subjects/{subject_id}",
            body=await async_maybe_transform(
                {
                    "email": email,
                    "metadata": metadata,
                    "name": name,
                },
                subject_update_params.SubjectUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubjectResource,
        )

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubjectListResponse:
        """
        List Subjects

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/subjects",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    subject_list_params.SubjectListParams,
                ),
            ),
            cast_to=SubjectListResponse,
        )

    async def delete(
        self,
        subject_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Subject

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subject_id:
            raise ValueError(f"Expected a non-empty value for `subject_id` but received {subject_id!r}")
        return await self._delete(
            f"/subjects/{subject_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class SubjectsResourceWithRawResponse:
    def __init__(self, subjects: SubjectsResource) -> None:
        self._subjects = subjects

        self.create = to_raw_response_wrapper(
            subjects.create,
        )
        self.retrieve = to_raw_response_wrapper(
            subjects.retrieve,
        )
        self.update = to_raw_response_wrapper(
            subjects.update,
        )
        self.list = to_raw_response_wrapper(
            subjects.list,
        )
        self.delete = to_raw_response_wrapper(
            subjects.delete,
        )


class AsyncSubjectsResourceWithRawResponse:
    def __init__(self, subjects: AsyncSubjectsResource) -> None:
        self._subjects = subjects

        self.create = async_to_raw_response_wrapper(
            subjects.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            subjects.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            subjects.update,
        )
        self.list = async_to_raw_response_wrapper(
            subjects.list,
        )
        self.delete = async_to_raw_response_wrapper(
            subjects.delete,
        )


class SubjectsResourceWithStreamingResponse:
    def __init__(self, subjects: SubjectsResource) -> None:
        self._subjects = subjects

        self.create = to_streamed_response_wrapper(
            subjects.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            subjects.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            subjects.update,
        )
        self.list = to_streamed_response_wrapper(
            subjects.list,
        )
        self.delete = to_streamed_response_wrapper(
            subjects.delete,
        )


class AsyncSubjectsResourceWithStreamingResponse:
    def __init__(self, subjects: AsyncSubjectsResource) -> None:
        self._subjects = subjects

        self.create = async_to_streamed_response_wrapper(
            subjects.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            subjects.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            subjects.update,
        )
        self.list = async_to_streamed_response_wrapper(
            subjects.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            subjects.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel

__all__ = ["CustomerPortalCreateSessionResponse"]


class CustomerPortalCreateSessionResponse(BaseModel):
    expires_at: datetime
    """The date and time the customer portal session expires."""

    subject_id: str
    """The ID of the subject for the customer portal session."""

    url: str
    """The URL to redirect to the customer portal session."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["SubscriptionListParams"]


class SubscriptionListParams(TypedDict, total=False):
    limit: int

    offset: int

    rate_card_id: Optional[str]
    """The ID of the rate card to list subscriptions for.

    Cannot be used with subject_id.
    """

    subject_id: Optional[str]
    """The ID or external ID of the subject to list subscriptions for.

    Cannot be used with rate_card_id.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import TypedDict

__all__ = ["SubjectCreateParams"]


class SubjectCreateParams(TypedDict, total=False):
    email: Optional[str]
    """The email of the subject. Must be a valid email address."""

    external_id: Optional[str]
    """The ID of the subject in your system.

    If provided, you may use pass it to the API in place of the subject ID. Must be
    unique.
    """

    metadata: Dict[str, str]
    """Additional metadata about the subject.

    You may use this to store any custom data about the subject.
    """

    name: Optional[str]
    """The name of the subject. Used for display in the dashboard."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, TypedDict

__all__ = ["SubjectUpdateParams"]


class SubjectUpdateParams(TypedDict, total=False):
    email: Required[Optional[str]]
    """The email of the subject. Must be a valid email address."""

    metadata: Required[Optional[Dict[str, str]]]
    """Additional metadata about the subject.

    You may use this to store any custom data about the subject.
    """

    name: Required[Optional[str]]
    """The name of the subject. Used for display in the dashboard."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CheckoutCallbackParam"]


class CheckoutCallbackParam(TypedDict, total=False):
    cancelled_url: Required[str]
    """The URL to redirect to after the checkout is cancelled."""

    success_url: Required[str]
    """The URL to redirect to after the checkout is successful."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SubscriptionResource", "CurrentPeriod"]


class CurrentPeriod(BaseModel):
    end: datetime

    start: datetime

    inclusive_end: Optional[bool] = None

    inclusive_start: Optional[bool] = None


class SubscriptionResource(BaseModel):
    id: str
    """The ID of the subscription."""

    cancels_at_end_of_cycle: bool
    """Whether the subscription will be cancelled at the end of the current cycle."""

    current_period: Optional[CurrentPeriod] = None
    """The current period of the subscription if it is active."""

    cycles_next_at: Optional[datetime] = None
    """The date and time the next cycle of the subscription will start."""

    effective_at: datetime
    """The date and time the subscription became effective."""

    metadata: Dict[str, str]

    rate_card_id: str
    """The ID of the rate card of the subscription."""

    status: Literal["active", "cancelled", "paused"]
    """The status of the subscription."""

    subject_id: str
    """The ID of the subject that the subscription is for."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PeriodParam"]


class PeriodParam(TypedDict, total=False):
    end: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    start: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    inclusive_end: bool

    inclusive_start: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UsageEventCreateParams"]


class UsageEventCreateParams(TypedDict, total=False):
    data: Required[Dict[str, Union[str, int]]]
    """The data of the usage event.

    This should contain any data that is needed to aggregate the usage event.
    """

    event_name: Required[str]
    """The name of the event.

    This is used by pricing metrics to aggregate usage events.
    """

    idempotency_key: Required[str]
    """The idempotency key for the usage event.

    This ensures that the same event is not processed multiple times.
    """

    subject_id: Required[str]
    """The ID or external ID of the subject that the usage event is for."""

    timestamp: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """The timestamp of the usage event.

    It is highly recommended to provide a timestamp. If not provided, the current
    timestamp will be used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .rate_card_resource import RateCardResource

__all__ = ["RateCardListResponse"]


class RateCardListResponse(BaseModel):
    has_more: bool

    rate_cards: List[RateCardResource]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import (
    pricing_metric_list_params,
    pricing_metric_create_params,
    pricing_metric_create_summary_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.period_param import PeriodParam
from ..types.pricing_metric_resource import PricingMetricResource
from ..types.pricing_metric_list_response import PricingMetricListResponse
from ..types.pricing_metric_create_summary_response import PricingMetricCreateSummaryResponse

__all__ = ["PricingMetricsResource", "AsyncPricingMetricsResource"]


class PricingMetricsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PricingMetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return PricingMetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PricingMetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return PricingMetricsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        aggregation: pricing_metric_create_params.Aggregation,
        event_name: str,
        name: str,
        unit: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PricingMetricResource:
        """
        Create Pricing Metric

        Args:
          aggregation: The aggregation function used to compute the value of the pricing metric.

          event_name: The name of the event that the pricing metric is computed on.

          name: The name of the pricing metric.

          unit: Unit of measurement for the pricing metric.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/pricing-metrics",
            body=maybe_transform(
                {
                    "aggregation": aggregation,
                    "event_name": event_name,
                    "name": name,
                    "unit": unit,
                },
                pricing_metric_create_params.PricingMetricCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingMetricResource,
        )

    def retrieve(
        self,
        pricing_metric_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PricingMetricResource:
        """
        Get Pricing Metric

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pricing_metric_id:
            raise ValueError(f"Expected a non-empty value for `pricing_metric_id` but received {pricing_metric_id!r}")
        return self._get(
            f"/pricing-metrics/{pricing_metric_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingMetricResource,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PricingMetricListResponse:
        """
        List Pricing Metrics

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/pricing-metrics",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"limit": limit}, pricing_metric_list_params.PricingMetricListParams),
            ),
            cast_to=PricingMetricListResponse,
        )

    def create_summary(
        self,
        pricing_metric_id: str,
        *,
        period: PeriodParam,
        subject_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PricingMetricCreateSummaryResponse:
        """
        Create Pricing Metric Summary

        Args:
          period: The period that the summary should be computed over.

          subject_id: The ID or external ID of the subject that the summary should be computed for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pricing_metric_id:
            raise ValueError(f"Expected a non-empty value for `pricing_metric_id` but received {pricing_metric_id!r}")
        return self._post(
            f"/pricing-metrics/{pricing_metric_id}/summary",
            body=maybe_transform(
                {
                    "period": period,
                    "subject_id": subject_id,
                },
                pricing_metric_create_summary_params.PricingMetricCreateSummaryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingMetricCreateSummaryResponse,
        )


class AsyncPricingMetricsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPricingMetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPricingMetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPricingMetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return AsyncPricingMetricsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        aggregation: pricing_metric_create_params.Aggregation,
        event_name: str,
        name: str,
        unit: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PricingMetricResource:
        """
        Create Pricing Metric

        Args:
          aggregation: The aggregation function used to compute the value of the pricing metric.

          event_name: The name of the event that the pricing metric is computed on.

          name: The name of the pricing metric.

          unit: Unit of measurement for the pricing metric.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/pricing-metrics",
            body=await async_maybe_transform(
                {
                    "aggregation": aggregation,
                    "event_name": event_name,
                    "name": name,
                    "unit": unit,
                },
                pricing_metric_create_params.PricingMetricCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingMetricResource,
        )

    async def retrieve(
        self,
        pricing_metric_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PricingMetricResource:
        """
        Get Pricing Metric

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pricing_metric_id:
            raise ValueError(f"Expected a non-empty value for `pricing_metric_id` but received {pricing_metric_id!r}")
        return await self._get(
            f"/pricing-metrics/{pricing_metric_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingMetricResource,
        )

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PricingMetricListResponse:
        """
        List Pricing Metrics

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/pricing-metrics",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"limit": limit}, pricing_metric_list_params.PricingMetricListParams),
            ),
            cast_to=PricingMetricListResponse,
        )

    async def create_summary(
        self,
        pricing_metric_id: str,
        *,
        period: PeriodParam,
        subject_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PricingMetricCreateSummaryResponse:
        """
        Create Pricing Metric Summary

        Args:
          period: The period that the summary should be computed over.

          subject_id: The ID or external ID of the subject that the summary should be computed for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not pricing_metric_id:
            raise ValueError(f"Expected a non-empty value for `pricing_metric_id` but received {pricing_metric_id!r}")
        return await self._post(
            f"/pricing-metrics/{pricing_metric_id}/summary",
            body=await async_maybe_transform(
                {
                    "period": period,
                    "subject_id": subject_id,
                },
                pricing_metric_create_summary_params.PricingMetricCreateSummaryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PricingMetricCreateSummaryResponse,
        )


class PricingMetricsResourceWithRawResponse:
    def __init__(self, pricing_metrics: PricingMetricsResource) -> None:
        self._pricing_metrics = pricing_metrics

        self.create = to_raw_response_wrapper(
            pricing_metrics.create,
        )
        self.retrieve = to_raw_response_wrapper(
            pricing_metrics.retrieve,
        )
        self.list = to_raw_response_wrapper(
            pricing_metrics.list,
        )
        self.create_summary = to_raw_response_wrapper(
            pricing_metrics.create_summary,
        )


class AsyncPricingMetricsResourceWithRawResponse:
    def __init__(self, pricing_metrics: AsyncPricingMetricsResource) -> None:
        self._pricing_metrics = pricing_metrics

        self.create = async_to_raw_response_wrapper(
            pricing_metrics.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            pricing_metrics.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            pricing_metrics.list,
        )
        self.create_summary = async_to_raw_response_wrapper(
            pricing_metrics.create_summary,
        )


class PricingMetricsResourceWithStreamingResponse:
    def __init__(self, pricing_metrics: PricingMetricsResource) -> None:
        self._pricing_metrics = pricing_metrics

        self.create = to_streamed_response_wrapper(
            pricing_metrics.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            pricing_metrics.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            pricing_metrics.list,
        )
        self.create_summary = to_streamed_response_wrapper(
            pricing_metrics.create_summary,
        )


class AsyncPricingMetricsResourceWithStreamingResponse:
    def __init__(self, pricing_metrics: AsyncPricingMetricsResource) -> None:
        self._pricing_metrics = pricing_metrics

        self.create = async_to_streamed_response_wrapper(
            pricing_metrics.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            pricing_metrics.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            pricing_metrics.list,
        )
        self.create_summary = async_to_streamed_response_wrapper(
            pricing_metrics.create_summary,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel
from .amount_output import AmountOutput

__all__ = ["PackagePriceOutput"]


class PackagePriceOutput(BaseModel):
    amount: AmountOutput

    package_units: int

    rounding_behavior: Literal["round_up", "round_down"]

    price_type: Optional[Literal["package"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["CustomerAccessRetrieveBillingStateResponse", "ActiveSubscription", "UsageData"]


class ActiveSubscription(BaseModel):
    rate_card_id: str

    subscription_id: str


class UsageData(BaseModel):
    included_units: int

    pricing_metric_id: str

    rate_name: str

    used_units: str


class CustomerAccessRetrieveBillingStateResponse(BaseModel):
    active_subscriptions: List[ActiveSubscription]
    """List of active subscriptions the subject is subscribed to."""

    has_active_subscription: bool
    """Whether the subject has an active subscription."""

    has_overage_for_usage: bool
    """
    Whether the subject has exceeded the included usage (if any) on a usage-based
    rate they are subscribed to.
    """

    usage_data: List[UsageData]
    """The usage data for the usage-based rates the subject is subscribed to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import customer_portal_create_session_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.customer_portal_create_session_response import CustomerPortalCreateSessionResponse

__all__ = ["CustomerPortalResource", "AsyncCustomerPortalResource"]


class CustomerPortalResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomerPortalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return CustomerPortalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomerPortalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return CustomerPortalResourceWithStreamingResponse(self)

    def create_session(
        self,
        *,
        return_url: str,
        subject_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerPortalCreateSessionResponse:
        """
        Create Customer Portal Session

        Args:
          return_url: The URL to redirect customers to if they click the back button on the customer
              portal.

          subject_id: The ID or external ID of the subject to create the customer portal session for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/customer-portal/sessions",
            body=maybe_transform(
                {
                    "return_url": return_url,
                    "subject_id": subject_id,
                },
                customer_portal_create_session_params.CustomerPortalCreateSessionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerPortalCreateSessionResponse,
        )


class AsyncCustomerPortalResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomerPortalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomerPortalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomerPortalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return AsyncCustomerPortalResourceWithStreamingResponse(self)

    async def create_session(
        self,
        *,
        return_url: str,
        subject_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerPortalCreateSessionResponse:
        """
        Create Customer Portal Session

        Args:
          return_url: The URL to redirect customers to if they click the back button on the customer
              portal.

          subject_id: The ID or external ID of the subject to create the customer portal session for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/customer-portal/sessions",
            body=await async_maybe_transform(
                {
                    "return_url": return_url,
                    "subject_id": subject_id,
                },
                customer_portal_create_session_params.CustomerPortalCreateSessionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerPortalCreateSessionResponse,
        )


class CustomerPortalResourceWithRawResponse:
    def __init__(self, customer_portal: CustomerPortalResource) -> None:
        self._customer_portal = customer_portal

        self.create_session = to_raw_response_wrapper(
            customer_portal.create_session,
        )


class AsyncCustomerPortalResourceWithRawResponse:
    def __init__(self, customer_portal: AsyncCustomerPortalResource) -> None:
        self._customer_portal = customer_portal

        self.create_session = async_to_raw_response_wrapper(
            customer_portal.create_session,
        )


class CustomerPortalResourceWithStreamingResponse:
    def __init__(self, customer_portal: CustomerPortalResource) -> None:
        self._customer_portal = customer_portal

        self.create_session = to_streamed_response_wrapper(
            customer_portal.create_session,
        )


class AsyncCustomerPortalResourceWithStreamingResponse:
    def __init__(self, customer_portal: AsyncCustomerPortalResource) -> None:
        self._customer_portal = customer_portal

        self.create_session = async_to_streamed_response_wrapper(
            customer_portal.create_session,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .flat_price_input_param import FlatPriceInputParam
from .package_price_input_param import PackagePriceInputParam

__all__ = ["RateCardCreateParams", "FixedRate", "FixedRatePrice", "UsageBasedRate", "UsageBasedRatePrice"]


class RateCardCreateParams(TypedDict, total=False):
    billing_interval: Required[Literal["monthly", "yearly"]]
    """How often the customer will be billed for this rate card."""

    name: Required[str]
    """The name of the rate card displayed to the customer."""

    description: Optional[str]
    """The description of the rate card displayed to the customer."""

    fixed_rates: Iterable[FixedRate]
    """The fixed rates of the rate card.

    These are billed at the start of each billing cycle.
    """

    metadata: Dict[str, str]

    usage_based_rates: Iterable[UsageBasedRate]
    """The usage based rates of the rate card.

    These are billed at the end of each billing cycle.
    """


FixedRatePrice: TypeAlias = Union[FlatPriceInputParam, PackagePriceInputParam]


class FixedRate(TypedDict, total=False):
    name: Required[str]
    """The name of the rate displayed to the customer."""

    price: Required[FixedRatePrice]
    """Flat price is a price that linearly scales with the quantity."""

    description: Optional[str]
    """The description of the rate displayed to the customer."""


UsageBasedRatePrice: TypeAlias = Union[FlatPriceInputParam, PackagePriceInputParam]


class UsageBasedRate(TypedDict, total=False):
    name: Required[str]
    """The name of the rate displayed to the customer."""

    price: Required[UsageBasedRatePrice]
    """Flat price is a price that linearly scales with the quantity."""

    pricing_metric_id: Required[str]
    """The ID of the pricing metric to use for this rate."""

    usage_based_rate_type: Required[Literal["simple"]]

    description: Optional[str]
    """The description of the rate displayed to the customer."""

    included_units: int
    """The number of units included in the rate before the price is applied."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .invoices import (
    InvoicesResource,
    AsyncInvoicesResource,
    InvoicesResourceWithRawResponse,
    AsyncInvoicesResourceWithRawResponse,
    InvoicesResourceWithStreamingResponse,
    AsyncInvoicesResourceWithStreamingResponse,
)
from .subjects import (
    SubjectsResource,
    AsyncSubjectsResource,
    SubjectsResourceWithRawResponse,
    AsyncSubjectsResourceWithRawResponse,
    SubjectsResourceWithStreamingResponse,
    AsyncSubjectsResourceWithStreamingResponse,
)
from .rate_cards import (
    RateCardsResource,
    AsyncRateCardsResource,
    RateCardsResourceWithRawResponse,
    AsyncRateCardsResourceWithRawResponse,
    RateCardsResourceWithStreamingResponse,
    AsyncRateCardsResourceWithStreamingResponse,
)
from .usage_events import (
    UsageEventsResource,
    AsyncUsageEventsResource,
    UsageEventsResourceWithRawResponse,
    AsyncUsageEventsResourceWithRawResponse,
    UsageEventsResourceWithStreamingResponse,
    AsyncUsageEventsResourceWithStreamingResponse,
)
from .subscriptions import (
    SubscriptionsResource,
    AsyncSubscriptionsResource,
    SubscriptionsResourceWithRawResponse,
    AsyncSubscriptionsResourceWithRawResponse,
    SubscriptionsResourceWithStreamingResponse,
    AsyncSubscriptionsResourceWithStreamingResponse,
)
from .customer_access import (
    CustomerAccessResource,
    AsyncCustomerAccessResource,
    CustomerAccessResourceWithRawResponse,
    AsyncCustomerAccessResourceWithRawResponse,
    CustomerAccessResourceWithStreamingResponse,
    AsyncCustomerAccessResourceWithStreamingResponse,
)
from .customer_portal import (
    CustomerPortalResource,
    AsyncCustomerPortalResource,
    CustomerPortalResourceWithRawResponse,
    AsyncCustomerPortalResourceWithRawResponse,
    CustomerPortalResourceWithStreamingResponse,
    AsyncCustomerPortalResourceWithStreamingResponse,
)
from .pricing_metrics import (
    PricingMetricsResource,
    AsyncPricingMetricsResource,
    PricingMetricsResourceWithRawResponse,
    AsyncPricingMetricsResourceWithRawResponse,
    PricingMetricsResourceWithStreamingResponse,
    AsyncPricingMetricsResourceWithStreamingResponse,
)

__all__ = [
    "CustomerPortalResource",
    "AsyncCustomerPortalResource",
    "CustomerPortalResourceWithRawResponse",
    "AsyncCustomerPortalResourceWithRawResponse",
    "CustomerPortalResourceWithStreamingResponse",
    "AsyncCustomerPortalResourceWithStreamingResponse",
    "RateCardsResource",
    "AsyncRateCardsResource",
    "RateCardsResourceWithRawResponse",
    "AsyncRateCardsResourceWithRawResponse",
    "RateCardsResourceWithStreamingResponse",
    "AsyncRateCardsResourceWithStreamingResponse",
    "UsageEventsResource",
    "AsyncUsageEventsResource",
    "UsageEventsResourceWithRawResponse",
    "AsyncUsageEventsResourceWithRawResponse",
    "UsageEventsResourceWithStreamingResponse",
    "AsyncUsageEventsResourceWithStreamingResponse",
    "SubscriptionsResource",
    "AsyncSubscriptionsResource",
    "SubscriptionsResourceWithRawResponse",
    "AsyncSubscriptionsResourceWithRawResponse",
    "SubscriptionsResourceWithStreamingResponse",
    "AsyncSubscriptionsResourceWithStreamingResponse",
    "SubjectsResource",
    "AsyncSubjectsResource",
    "SubjectsResourceWithRawResponse",
    "AsyncSubjectsResourceWithRawResponse",
    "SubjectsResourceWithStreamingResponse",
    "AsyncSubjectsResourceWithStreamingResponse",
    "PricingMetricsResource",
    "AsyncPricingMetricsResource",
    "PricingMetricsResourceWithRawResponse",
    "AsyncPricingMetricsResourceWithRawResponse",
    "PricingMetricsResourceWithStreamingResponse",
    "AsyncPricingMetricsResourceWithStreamingResponse",
    "CustomerAccessResource",
    "AsyncCustomerAccessResource",
    "CustomerAccessResourceWithRawResponse",
    "AsyncCustomerAccessResourceWithRawResponse",
    "CustomerAccessResourceWithStreamingResponse",
    "AsyncCustomerAccessResourceWithStreamingResponse",
    "InvoicesResource",
    "AsyncInvoicesResource",
    "InvoicesResourceWithRawResponse",
    "AsyncInvoicesResourceWithRawResponse",
    "InvoicesResourceWithStreamingResponse",
    "AsyncInvoicesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .subscription_resource import SubscriptionResource

__all__ = [
    "SubscriptionCreateResponse",
    "Result",
    "ResultCreateSubscriptionRequiresActionResponse",
    "ResultCreateSubscriptionRequiresActionResponseAction",
    "ResultCreateSubscriptionSuccessResponse",
]


class ResultCreateSubscriptionRequiresActionResponseAction(BaseModel):
    checkout_url: str
    """The URL of the checkout page to redirect to in order to complete the request."""

    requires_action_type: Literal["checkout"]


class ResultCreateSubscriptionRequiresActionResponse(BaseModel):
    action: ResultCreateSubscriptionRequiresActionResponseAction
    """The action to take to complete the request."""

    result_type: Literal["requires_action"]


class ResultCreateSubscriptionSuccessResponse(BaseModel):
    result_type: Literal["success"]

    subscription: SubscriptionResource
    """The created subscription resource."""


Result: TypeAlias = Annotated[
    Union[ResultCreateSubscriptionRequiresActionResponse, ResultCreateSubscriptionSuccessResponse],
    PropertyInfo(discriminator="result_type"),
]


class SubscriptionCreateResponse(BaseModel):
    result: Result
    """The result of the request.

    If the request is successful, the subscription resource will be returned. If the
    request is requires action, the action to take to complete the request will be
    returned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .amount_output import AmountOutput

__all__ = ["InvoiceListResponse", "Invoice", "InvoiceLineItem"]


class InvoiceLineItem(BaseModel):
    amount: AmountOutput

    description: str

    price_in_unit_amount: AmountOutput

    quantity: int


class Invoice(BaseModel):
    id: str
    """The ID of the invoice."""

    created_at: datetime
    """The date and time the invoice was created."""

    hosted_url: Optional[str] = None
    """The URL of the hosted invoice."""

    line_items: List[InvoiceLineItem]
    """The line items of the invoice."""

    status: Literal["draft", "open", "paid", "uncollectible", "void"]
    """The status of the invoice."""

    subject_id: str
    """The ID of the subject for the invoice."""

    total_amount: AmountOutput
    """The total amount of the invoice."""


class InvoiceListResponse(BaseModel):
    has_more: bool

    invoices: List[Invoice]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CustomerPortalCreateSessionParams"]


class CustomerPortalCreateSessionParams(TypedDict, total=False):
    return_url: Required[str]
    """
    The URL to redirect customers to if they click the back button on the customer
    portal.
    """

    subject_id: Required[str]
    """The ID or external ID of the subject to create the customer portal session for."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["AmountOutput"]


class AmountOutput(BaseModel):
    currency_code: str
    """The currency code of the amount."""

    value: str
    """The value of the amount in the smallest unit of the currency."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal

import httpx

from ..types import rate_card_list_params, rate_card_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.rate_card_resource import RateCardResource
from ..types.rate_card_list_response import RateCardListResponse

__all__ = ["RateCardsResource", "AsyncRateCardsResource"]


class RateCardsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RateCardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return RateCardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RateCardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return RateCardsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        billing_interval: Literal["monthly", "yearly"],
        name: str,
        description: Optional[str] | Omit = omit,
        fixed_rates: Iterable[rate_card_create_params.FixedRate] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        usage_based_rates: Iterable[rate_card_create_params.UsageBasedRate] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardResource:
        """
        Create Rate Card

        Args:
          billing_interval: How often the customer will be billed for this rate card.

          name: The name of the rate card displayed to the customer.

          description: The description of the rate card displayed to the customer.

          fixed_rates: The fixed rates of the rate card. These are billed at the start of each billing
              cycle.

          usage_based_rates: The usage based rates of the rate card. These are billed at the end of each
              billing cycle.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/rate-cards",
            body=maybe_transform(
                {
                    "billing_interval": billing_interval,
                    "name": name,
                    "description": description,
                    "fixed_rates": fixed_rates,
                    "metadata": metadata,
                    "usage_based_rates": usage_based_rates,
                },
                rate_card_create_params.RateCardCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardResource,
        )

    def retrieve(
        self,
        rate_card_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardResource:
        """
        Get Rate Card

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not rate_card_id:
            raise ValueError(f"Expected a non-empty value for `rate_card_id` but received {rate_card_id!r}")
        return self._get(
            f"/rate-cards/{rate_card_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardResource,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardListResponse:
        """
        List Rate Cards

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/rate-cards",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    rate_card_list_params.RateCardListParams,
                ),
            ),
            cast_to=RateCardListResponse,
        )


class AsyncRateCardsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRateCardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/uselark/lark-billing-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRateCardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRateCardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/uselark/lark-billing-python#with_streaming_response
        """
        return AsyncRateCardsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        billing_interval: Literal["monthly", "yearly"],
        name: str,
        description: Optional[str] | Omit = omit,
        fixed_rates: Iterable[rate_card_create_params.FixedRate] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        usage_based_rates: Iterable[rate_card_create_params.UsageBasedRate] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardResource:
        """
        Create Rate Card

        Args:
          billing_interval: How often the customer will be billed for this rate card.

          name: The name of the rate card displayed to the customer.

          description: The description of the rate card displayed to the customer.

          fixed_rates: The fixed rates of the rate card. These are billed at the start of each billing
              cycle.

          usage_based_rates: The usage based rates of the rate card. These are billed at the end of each
              billing cycle.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/rate-cards",
            body=await async_maybe_transform(
                {
                    "billing_interval": billing_interval,
                    "name": name,
                    "description": description,
                    "fixed_rates": fixed_rates,
                    "metadata": metadata,
                    "usage_based_rates": usage_based_rates,
                },
                rate_card_create_params.RateCardCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardResource,
        )

    async def retrieve(
        self,
        rate_card_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardResource:
        """
        Get Rate Card

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not rate_card_id:
            raise ValueError(f"Expected a non-empty value for `rate_card_id` but received {rate_card_id!r}")
        return await self._get(
            f"/rate-cards/{rate_card_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RateCardResource,
        )

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RateCardListResponse:
        """
        List Rate Cards

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/rate-cards",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    rate_card_list_params.RateCardListParams,
                ),
            ),
            cast_to=RateCardListResponse,
        )


class RateCardsResourceWithRawResponse:
    def __init__(self, rate_cards: RateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = to_raw_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = to_raw_response_wrapper(
            rate_cards.retrieve,
        )
        self.list = to_raw_response_wrapper(
            rate_cards.list,
        )


class AsyncRateCardsResourceWithRawResponse:
    def __init__(self, rate_cards: AsyncRateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = async_to_raw_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            rate_cards.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            rate_cards.list,
        )


class RateCardsResourceWithStreamingResponse:
    def __init__(self, rate_cards: RateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = to_streamed_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            rate_cards.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            rate_cards.list,
        )


class AsyncRateCardsResourceWithStreamingResponse:
    def __init__(self, rate_cards: AsyncRateCardsResource) -> None:
        self._rate_cards = rate_cards

        self.create = async_to_streamed_response_wrapper(
            rate_cards.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            rate_cards.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            rate_cards.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["Period"]


class Period(BaseModel):
    end: datetime

    start: datetime

    inclusive_end: Optional[bool] = None

    inclusive_start: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["SubjectResource"]


class SubjectResource(BaseModel):
    id: str
    """The ID of the subject."""

    created_at: datetime
    """The date and time the subject was created."""

    email: Optional[str] = None
    """The email of the subject."""

    external_id: Optional[str] = None
    """The ID of the subject in your system.

    You may pass it to the API in place of the subject ID.
    """

    metadata: Dict[str, str]
    """Additional metadata about the subject.

    You may use this to store any custom data about the subject.
    """

    name: Optional[str] = None
    """The name of the subject. Used for display in the dashboard."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = [
    "PricingMetricResource",
    "Aggregation",
    "AggregationSumAggregationPricingMetricResource",
    "AggregationCountAggregationPricingMetricResource",
    "AggregationMaxAggregationPricingMetricResource",
    "AggregationLastAggregationPricingMetricResource",
]


class AggregationSumAggregationPricingMetricResource(BaseModel):
    aggregation_type: Literal["sum"]

    value_field: str
    """The field to sum over."""


class AggregationCountAggregationPricingMetricResource(BaseModel):
    aggregation_type: Literal["count"]


class AggregationMaxAggregationPricingMetricResource(BaseModel):
    aggregation_type: Literal["max"]

    value_field: str
    """The field to get the max value of."""


class AggregationLastAggregationPricingMetricResource(BaseModel):
    aggregation_type: Literal["last"]

    value_field: str
    """The field to get the last value of."""


Aggregation: TypeAlias = Annotated[
    Union[
        AggregationSumAggregationPricingMetricResource,
        AggregationCountAggregationPricingMetricResource,
        AggregationMaxAggregationPricingMetricResource,
        AggregationLastAggregationPricingMetricResource,
    ],
    PropertyInfo(discriminator="aggregation_type"),
]


class PricingMetricResource(BaseModel):
    id: str
    """The ID of the pricing metric."""

    aggregation: Aggregation
    """The aggregation function used to compute the value of the pricing metric."""

    event_name: str
    """The event name that the pricing metric is computed on."""

    name: str
    """The name of the pricing metric."""

    unit: str
    """The unit of the value computed by the pricing metric."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .amount_input_param import AmountInputParam

__all__ = ["FlatPriceInputParam"]


class FlatPriceInputParam(TypedDict, total=False):
    amount: Required[AmountInputParam]

    price_type: Literal["flat"]
